//---------------------------------------------------------------------------
// $Id: main.cc,v 1.2 1998/06/03 20:09:21 diego Exp diego $
//
// Copyright (C) 1997 University of Alberta
//
// Author: Diego Novillo (diego@cs.ualberta.ca)
//
// This software is provided under the terms described in the
// "suif_copyright.h" include file.
//---------------------------------------------------------------------------
#include <suif_copyright.h>

#include <stdio.h>

#include <suif.h>
#include <par.h>
#include <pfg.h>
#include <cssame.h>
#include <odc-util.h>

FILE *STDOUT;
    

void 
do_proc(tree_proc *tp)
{
    proc_sym *psym = tp->proc();

    printf("------------------------------------------------------------------------------\n");
    printf("Analyzing function %s\n", psym->name());
    printf("------------------------------------------------------------------------------\n");

    pfg *graph = new pfg(tp, TRUE, FALSE, FALSE);
    graph->remove_unreachable_blocks();

    cssame *ssa = new cssame(graph);
    ssa->print();

    printf("\n\n");
    printf("------------------------------------------------------------------------------\n");
    printf("Reaching definitions for every use\n");
    printf("------------------------------------------------------------------------------\n");

    VarRef_list_iter iter(graph->refs());
    while (!iter.is_empty()) {
	VarRef *ref = iter.step();

	if (ref->isU()) {
	    printf("Reaching definitions for ");
	    ref->print();
	    printf("\n");

	    VarRef_list_iter rd_iter(ref->reachingDefs());
	    while (!rd_iter.is_empty()) {
		printf("\t");
		rd_iter.step()->print();
		printf("\n");
	    }
	    printf("\n");
	}
    }

    printf("\n\n");
    printf("------------------------------------------------------------------------------\n");
    printf("Def-use links\n");
    printf("------------------------------------------------------------------------------\n");
    VarRef_list_iter refs_iter(graph->refs());
    while (!refs_iter.is_empty()) {
	VarRef *ref = refs_iter.step();
	if (!ref->isD()) {
	    continue;
	}

	printf("Reached uses for ");
	ref->print();
	printf("\n");

	VarRef_list_iter uses_iter(ref->reachedUses());
	while (!uses_iter.is_empty()) {
	    uses_iter.step()->print();
	    printf("\n");
	}
	printf("\n");
    }
		

    printf("------------------------------------------------------------------------------\n\n");
}


main(int argc, char *argv[])
{
    LIBRARY(par, init_par, exit_par);
    LIBRARY(pfg, init_pfg, exit_pfg);
    LIBRARY(cssame, init_cssame, exit_cssame);

    init_suif(argc, argv);

    STDOUT = stdout;

    suif_proc_iter(argc, argv, do_proc);

    return 0;
}
