//---------------------------------------------------------------------------
// $Id: cssame.h,v 1.1 1998/06/03 16:52:43 diego Exp diego $
//
// Copyright (C) 1997 University of Alberta
//
// Author: Diego Novillo (diego@cs.ualberta.ca)
//
// This software is provided under the terms described in the
// "suif_copyright.h" include file.
//---------------------------------------------------------------------------
#ifndef __CSSAME_H
#define __CSSAME_H

#include <suif_copyright.h>

#ifdef CSSAMELIB
#    include "mutex-analysis.h"
#else
#    include <mutex-analysis.h>
#endif

/* Annotations used */
extern char *k_reaching_defs;	// Definitions reaching the statement
extern char *k_reached_uses;	// Uses reached by the statement
extern char *k_currdef;		// Current definition for a variable
extern char *k_mutex_struct;	// Mutex structure for a synch variable

class pfg;

//---------------------------------------------------------------------------
// class cssame
// Computes the CSSAME form for a parallel flow graph.
//---------------------------------------------------------------------------
class cssame {
protected:
    pfg *_graph;
    var_sym_list *_mutexVars;

    /* Internal methods used in the creation of parallel FUD chains */
    void buildCSSAME();
    void computeOrdering();
    StrictInterval_list *findMutexBodies();
    StrictInterval_list *findStrictIntervals(var_sym_list *vars_set, 
	    bit_set_list *pn_set, bit_set_list *px_set);
    void validateMutexBodies(bit_set_list *pn_set, bit_set_list *px_set);
    void placePhiFunctions();
    void placeFUDChains();
    void searchFUDChains(pfg_node *n);
    void placePiFunctions();
    void rewritePiFunctions();

    VarRef_list *sortPhiPi();

    /* Helper method to compute reaching definitions */
    void followChain(VarRef *d, VarRef *u);

public:
    cssame(pfg *graph) : _graph(graph) { buildCSSAME(); }

    pfg *graph() const { return _graph; }
    var_sym_list *mutexVars() const { return _mutexVars; }

    void computeReachingDefs();
    void print(FILE *f = stdout);
};
#endif	// __CSSAME_H
