/*  Control Flow Graph Utilities */

/*  Copyright (c) 1994 Stanford University

    All rights reserved.

    Copyright (c) 1995, 1996, 1997 The President and Fellows of Harvard 
    University

    All rights reserved.

    This software is provided under the terms described in
    the "suif_copyright.h" include file.

    Modified by Diego Novillo (diego@cs.ualberta.ca)
*/

#include <suif_copyright.h>


#ifndef PFG_UTIL_H
#define PFG_UTIL_H

/*  initialization and finalization functions */
void init_pfg(int &argc, char *argv[]);
void exit_pfg();

/*  functions to find PFG nodes associated with various objects */
pfg_node *label_pfg_node(label_sym *l);
pfg_node *pfg_begin_node(tree_node *n);
pfg_node *pfg_end_node(tree_node *n);
pfg_node *pfg_holding_node(tree_node *n);
pfg_test *pfg_test_node(tree_for *n);
pfg_label *pfg_toplab(tree_for *n);

/*  annotations to attach PFGs to SUIF objects */
extern char *k_pfg_node;
extern char *k_pfg_begin;
extern char *k_pfg_end;
extern char *k_pfg_test;
extern char *k_pfg_toplab;
extern char *k_proc_entry;

/* Annotations for data references and synchronization instructions */
extern char *k_refs;
extern char *k_defs;
extern char *k_uses;
extern char *k_phi_terms;
extern char *k_pi_terms;
extern char *k_head_sync;
extern char *k_tail_sync;
extern char *k_mutex_sync;
extern char *k_entry_sync_part;
extern char *k_exit_sync_part;

/* Various neat functions added */
extern void generate_vcg(FILE *f, pfg *pfg);    /* Dump vcg format file   */

/* Iterator over instructions in a pfg_block */
class pfg_node_instr_iter {
  private:
    tree_node_list_e *first;
    tree_node_list_e *last;
    tree_node_list_e *sentinel;
    tree_node_list_e *cur;
    tree_node_list_e *nxt;
    boolean rev; 

  public:
    pfg_node_instr_iter(pfg_node *node, boolean reverse=FALSE, boolean
	    exec=FALSE);

    void reset();
    tree_instr *step();
    tree_instr *peek();

    boolean is_empty();
};

/* Helper functions */
extern boolean occurs(pfg_node *cn, pfg_node_list *cnl); 
extern tree_instr *first_ti(tree_node_list *tnl);
extern tree_instr *last_ti(tree_node_list *tnl);


/* Functions from machineUtil.{cc,h} from machsuif */
boolean Is_label(instruction *i);
label_sym * Get_label(instruction *i);
boolean Is_ubr(instruction *i);
boolean Is_null(instruction *i);
boolean Is_cbr(instruction *i);
boolean Is_call(instruction *i);
boolean Is_mbr(instruction *i);
boolean Is_return(instruction *i);
boolean Is_cti(instruction *i);
sym_node * Get_target(instruction *i);

#endif /* PFG_UTIL_H */
