//---------------------------------------------------------------------------
// $Id: odc-util.cc,v 1.6 1998/06/03 16:52:35 diego Exp diego $
//
// Copyright (C) 1997 University of Alberta
//
// Author: Diego Novillo (diego@cs.ualberta.ca)
//
// This software is provided under the terms described in the
// "suif_copyright.h" include file.
//---------------------------------------------------------------------------
#include <suif_copyright.h>

#include <stdio.h>
#include <stdlib.h>
#include <str.h>
#include <regexp.h>

#include "odc-util.h"

static char _lastline[MAX_LINE_SIZE + 1];

//---------------------------------------------------------------------------
// displayFile()
// Outputs a range of lines of a file to stdout. Returns the last line
// read as a pointer to a static array of characters.
//---------------------------------------------------------------------------
char *
displayFile(char *fname, int start, int end, unsigned flags, FILE *to)
{
    String str(file2String(fname, start, end, flags));

    fprintf(to, "%s", str());

    return _lastline;
}


//---------------------------------------------------------------------------
// file2String()
// Reads a file into a string and returns it. It also sets the local
// variable _lastline to the last line read from the file.
//---------------------------------------------------------------------------
String&
file2String(char *fname, int start, int end, unsigned flags)
{
    static char line[MAX_LINE_SIZE + 1];
    int lineno = 1;
    FILE *f;
    String str;
    String str_line;
    
    f = fopen(fname, "r");
    if (f == NULL) {
	perror(fname);
	exit(-1);
    }

    /* Skip the first 'start' lines */
    while (lineno < start) {
	fgets(line, MAX_LINE_SIZE, f);
	lineno++;
    }

    /* Read lines between 'start' and 'end'. */
    while (lineno <= end && !feof(f)) {
	fgets(line, MAX_LINE_SIZE, f);
	str_line = line;

	if (!feof(f)) {
	    if (flags & DF_LINENO) {
		char str_lineno[8];
		sprintf(str_lineno, "%7d", lineno);
		str_line.Insert(0, str_lineno);
	    }

	    if (flags & DF_QUOTES) {
		str_line.Replace("\"", "'");
	    }

	    if (flags & DF_LTRIM) {
		str_line.Trim(String::LEFT);
	    }

	    if (flags & DF_RTRIM) {
		str_line.Trim(String::RIGHT);
	    }

	    if (flags & DF_TRIMLAST && lineno == end) {
		str_line.Replace("\n", "");
	    }

	    if (flags & DF_XLATE) {
		str_line.Replace("\n", "\\n");
	    }

	    str += str_line;
	    lineno++;
	}
    }

    fclose(f);

    if (str_line()) {
	strcpy(_lastline, str_line());
    } else {
	_lastline[0] = '\0';
    }

    return str;
}


//---------------------------------------------------------------------------
// debug()
// Returns TRUE if the environment variable ODC_DEBUG contains the
// given string and its level is greater or equal than the given level.
// The format of the debug variable is number|(number:name)+
// For example: 
// $ setenv ODC_DEBUG 2-foo-1-bar
//	sets 'foo' to debugging level 2 and 'bar' to debugging level 1.
//
// $ setenv ODC_DEBUG 3
//	sets every function to debugging level 3.
//---------------------------------------------------------------------------
int 
debug(char *fname, int level)
{
    char *env = getenv("ODC_DEBUG");

    if (env == NULL) return 0;

    String *fields;
    String str_debug(env);
    int i, nf = split(str_debug, fields, "-");

    if (nf == 0 && atoi(str_debug) >= level) {
	return 1;
    }

    if (nf % 2 == 0) {
	for (i = 0; i < nf - 1; i += 2) {
	    RegExp re(fields[i + 1]);
	    int m = match(fname, re);
	    if (atoi(fields[i]) >= level && m >= 0) {
		return 1;
	    }
	}
    } else {
	printf("debug(): Error parsing env variable PFGLIB_DEBUG.\n");
	printf("         Format is number|(number-name)+\n");
	exit(1);
    }

    return 0;
}
